<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/wordpress-form-wizard/
 * @since      1.0.0
 *
 * @package    WordPress_Form_Wizard_Public
 * @subpackage WordPress_Form_Wizard
 */
class WordPress_Form_Wizard_Public extends WordPress_Form_Wizard {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * options of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $options
	 */
	protected $options;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
    public function __construct($plugin_name, $version)
    {
        $this->plugin_name = $plugin_name;
        $this->version = $version;

		$this->prefix = "wordpress_form_wizard_";
		
	}

	/**
	 * Enqueu Styles
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://welaunch.io
	 * @return  [type]                       [description]
	 */
	public function enqueue_scripts_styles() 
	{
		global $wordpress_form_wizard_options;
		$this->options = $wordpress_form_wizard_options;

		if (!$this->get_option('enable')) {
			return false;
		}

		wp_enqueue_style( 'boxicons', plugin_dir_url( dirname( __FILE__ ) ) . 'assets/vendor/boxicons/css/boxicons.min.css', array(), '2.1.4', 'all' );
		wp_enqueue_style( $this->plugin_name . '-public', plugin_dir_url( dirname( __FILE__ ) ) . 'assets/css/wordpress-form-wizard-public.css', array(), $this->version, 'all' );

		// Yes we only used ONE external JS library
		wp_enqueue_script('rangeslider', plugin_dir_url( dirname( __FILE__ ) ) . 'assets/vendor/rangeslider/rangeslider.min.js', array('jquery'), '2.3.0', true);
		wp_enqueue_script($this->plugin_name . '-public', plugin_dir_url( dirname( __FILE__ ) ) . 'assets/js/wordpress-form-wizard-public.js', array('jquery'), $this->version, true);

		$forJS = array(
			'ajax_url' => admin_url('admin-ajax.php'),
			'numberFormatterLocale' => $this->get_option('numberFormatterLocale'),
			'numberFormatterStyle' => $this->get_option('numberFormatterStyle'),
			'numberFormatterCurrency' => $this->get_option('numberFormatterCurrency'),
			'scrollToTop' => $this->get_option('scrollToTop'),
			'scrollToTopOffset' => $this->get_option('scrollToTopOffset'),
			'resultsLoadMore' => $this->get_option('resultsLoadMore'),
		);
        $forJS = apply_filters('wordpress_form_wizard', $forJS);
        
        wp_localize_script($this->plugin_name . '-public', 'form_wizard_options', $forJS);
	}

	
	public function init()
	{
		global $wordpress_form_wizard_options;
		$this->options = $wordpress_form_wizard_options;

		$this->allowed_tags = wp_kses_allowed_html('post');

        add_shortcode( 'wordpress_form_wizard', array($this, 'shortcode'));
        add_shortcode( 'wordpress_kitchen_price_calculator', array($this, 'shortcode'));
        
        add_action('wp_ajax_nopriv_wordpress_form_wizard_get_posts', array($this, 'get_posts') );
        add_action('wp_ajax_wordpress_form_wizard_get_posts', array($this, 'get_posts') );

		if(isset($_GET['pdf-catalog'])) {
			add_filter('option_woocommerce_attribute_lookup_enabled', array($this, 'maybe_disable_wc_attribute_lookup_table'), 99, 2);
		}
    }

    public function maybe_disable_wc_attribute_lookup_table($value, $option)
    {
    	return 'no';
    }

    public function get_posts()
    {
       $response = array(
            'status' => 0,
            'message' => '',
            'data' => array(),
            'post_ids' => array(),
        );

        if (!defined('DOING_AJAX') || !DOING_AJAX) {
            $response['message'] = esc_html__('No AJAX call', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['form_id']) || empty($_POST['form_id'])) {
            $response['message'] = esc_html__('Form id missing.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

        if (!isset($_POST['choices_made']) || empty($_POST['choices_made'])) {
            $response['message'] = esc_html__('No choices made.', 'wordpress-form-wizard');
            echo json_encode($response);
            die();
        }

		$form_id = (int) $_POST['form_id'];
		$choices_made = array_filter( (array) $_POST['choices_made'] );

		$order = $this->get_option('order');
		$orderby = $this->get_option('orderby');
		$orderbyCustomPostMeta = $this->get_option('orderbyCustomPostMeta');

		// Setup Ordering
		$orderbyCustomPostMeta = ""; 
		if(empty($orderby)) {
			$orderby = "title";
		}

		$needMetaKey = array( '_price', '_regular_price', '_sale_price', '_stock');
		if(in_array($orderby, $needMetaKey)) {
			$orderbyCustomPostMeta = $orderby;
			$orderby = 'meta_value_num';
		}

		$needMetaKey = array('_sku');
		if(in_array($orderby, $needMetaKey)) {
			$orderbyCustomPostMeta = $orderby;
			$orderby = 'meta_value';
		}

		if($orderby == "lowestMidHighestPrice") {
			$orderbyCustomPostMeta = '_price';
			$orderby = 'meta_value_num';
		}

		if($orderby == 'custom_post_meta') {
			$orderbyCustomPostMeta = $this->get_option('orderbyCustomPostMeta');
			if($this->get_option('orderbyCustomPostMetaNumeric')) {
				$orderby = 'meta_value_num';
			} else {
				$orderby = 'meta_value';	
			}
		}

		if(empty($order)) {
			$order = "DESC";
		}

		$args = array(
			'numberposts' => '-1',
			'order' => $order,
			'orderby' => $orderby,
			'meta_key' => $orderbyCustomPostMeta,
		);

		$tax_query = array(
			'relation' => 'AND',
		);

		$meta_query = array(
			'relation' => 'AND',
		);

		$niceToHave = array(

		);

		$form_post_types = get_post_meta($form_id, $this->prefix . 'post_types', true);
		if(!empty($form_post_types) && is_array($form_post_types)) {
			$args['post_type'] = $form_post_types;

			if(in_array('product', $form_post_types) || in_array('product_variation', $form_post_types)) {
				$tax_query[] = array(
		            'taxonomy'  => 'product_visibility',
		            'terms'     => array('exclude-from-catalog'),
		            'field'     => 'name',
		            'operator'  => 'NOT IN',
		        );

		        if($this->get_option('resultsHideOutOfStock')) {
		        	$meta_query[] = array(
			            'key' => '_stock_status',
			            'value' => 'instock',
			            'compare' => '=',
		            );
		        }
	        }
		} else {
			$args['post_type'] = 'any';
		}


		$post_in = array();
		if(!empty($choices_made)) {

			foreach($choices_made as $questionId => $answerIds) {
				
				if(empty($answerIds) || !is_array($answerIds)) {
					continue;
				}

				$type = get_post_meta($questionId, $this->prefix . 'type', true);
				$filtering = get_post_meta($questionId, $this->prefix . 'filtering', true);
				$relation = get_post_meta($questionId, $this->prefix . 'relation', true);

				if($type == "slider") {

					$sliderTaxonomyName = get_post_meta($questionId, $this->prefix . 'slider_taxonomy_name', true);
					$sliderTaxonomyOperator = get_post_meta($questionId, $this->prefix . 'slider_taxonomy_operator', true);
					if(!empty($sliderTaxonomyName)) {

						if(empty($sliderTaxonomyOperator)) {
							$sliderTaxonomyOperator = 'IN';
						}

						if($sliderTaxonomyOperator == ">=" || $sliderTaxonomyOperator == ">" || $sliderTaxonomyOperator == "<=" || $sliderTaxonomyOperator == "<") {

							$termToCheckValue = floatval( reset( $answerIds ) );
							$allTerms = get_terms( array(
								'taxonomy' => $sliderTaxonomyName
							) );

							$tmp = array();
							if(!empty($allTerms)) {

								foreach($allTerms as $allTerm) {
									$allTermValue = floatval( $allTerm->name );
									if(!empty($allTermValue) ) {
										if($sliderTaxonomyOperator == ">=" && $allTermValue >= $termToCheckValue) {
											$tmp[] = $allTerm->term_id;
										} elseif ($sliderTaxonomyOperator == ">" && $allTermValue > $termToCheckValue) {
											$tmp[] = $allTerm->term_id;
										} elseif ($sliderTaxonomyOperator == "<" && $allTermValue < $termToCheckValue) {
											$tmp[] = $allTerm->term_id;
										} elseif ($sliderTaxonomyOperator == "<=" && $allTermValue <= $termToCheckValue) {
											$tmp[] = $allTerm->term_id;
										}
									}
								}
							}

							$answerIds = $tmp;
							$sliderTaxonomyOperator = 'IN';
						}

						if($filtering == "ranking") {

							$niceToHave[$questionId][0] = array(
				                'taxonomy' => $sliderTaxonomyName,
				                'field' => 'term_id',
				                'terms' => $answerIds,
				                'operator' => $sliderTaxonomyOperator,
							);

						} else {
							$tax_query[$questionId][] = array(
				                'taxonomy' => $sliderTaxonomyName,
				                'field' => 'term_id',
				                'terms' => $answerIds,
				                'operator' => $sliderTaxonomyOperator,
							);
						}

					}

					$sliderMetaKey = get_post_meta($questionId, $this->prefix . 'slider_meta_key', true);
					$sliderMetaOperator = get_post_meta($questionId, $this->prefix . 'slider_meta_operator', true);
					$sliderMetaType = get_post_meta($questionId, $this->prefix . 'slider_meta_type', true);
					if(!empty($sliderMetaKey)) {

						if($filtering == "ranking") {

							$niceToHave[$questionId][0] = array(
				                'key' => $sliderMetaKey,
				                'compare' => !empty($sliderMetaOperator) ? $sliderMetaOperator : '=',
				                'value' => floatval( reset( $answerIds ) ),
				                'type' => !empty($sliderMetaType) ? $sliderMetaType : 'CHAR',
							);

						} else {

							$meta_query[$questionId][] = array(
				                'key' => $sliderMetaKey,
				                'compare' => !empty($sliderMetaOperator) ? $sliderMetaOperator : '=',
				                'value' => floatval( reset( $answerIds ) ),
				                'type' => !empty($sliderMetaType) ? $sliderMetaType : 'CHAR',
							);

						}
					}

					continue;

				}

				// sliderTaxonomyName
				// sliderTaxonomyOperator

				foreach($answerIds as $answerId) {

					$taxonomies = get_post_meta($answerId, $this->prefix . 'taxonomies', true);	


					if(!empty($taxonomies)) {

						foreach($taxonomies as $taxonomy) {

							if(!isset($taxonomy['name']) || empty($taxonomy['name']) || !isset($taxonomy['values']) || empty($taxonomy['values'])) {
								continue;
							}

							if($filtering == "ranking") {

								if(!isset($niceToHave[$questionId])) {
									$niceToHave[$questionId] = array();
								}

							} else {

								if(!isset($tax_query[$questionId])) {

									$tax_query[$questionId] = array(
										'relation' => $relation,
									);
								}
							}

							
							$operator = $taxonomy['operator'];
							if($operator == ">=" || $operator == ">" || $operator == "<=" || $operator == "<") {

								$termToCheck = get_term(reset( $taxonomy['values'] ), $taxonomy['name']);
								if(!isset($termToCheck->name)) {
									continue;	
								}

								$termToCheckValue = floatval( $termToCheck->name );

								$allTerms = get_terms( array(
									'taxonomy' => $taxonomy['name']
								) );

								$tmp = array();
								if(!empty($allTerms)) {

									foreach($allTerms as $allTerm) {
										$allTermValue = floatval( $allTerm->name );
										if(!empty($allTermValue) ) {
											if($operator == ">=" && $allTermValue >= $termToCheckValue) {
												$tmp[] = $allTerm->term_id;
											} elseif ($operator == ">" && $allTermValue > $termToCheckValue) {
												$tmp[] = $allTerm->term_id;
											} elseif ($operator == "<" && $allTermValue < $termToCheckValue) {
												$tmp[] = $allTerm->term_id;
											} elseif ($operator == "<=" && $allTermValue <= $termToCheckValue) {
												$tmp[] = $allTerm->term_id;
											}
										}
									}
								}

								$taxonomy['values'] = $tmp;
								$taxonomy['operator'] = 'IN';
							}

							
							if($filtering == "ranking") {

								$niceToHave[$questionId][$answerId] = array(
					                'taxonomy' => $taxonomy['name'],
					                'field' => 'term_id',
					                'terms' => $taxonomy['values'],
					                'operator' => !empty($taxonomy['operator']) ? $taxonomy['operator'] : 'IN',
								);

							} else {
								$tax_query[$questionId][] = array(
					                'taxonomy' => $taxonomy['name'],
					                'field' => 'term_id',
					                'terms' => $taxonomy['values'],
					                'operator' => !empty($taxonomy['operator']) ? $taxonomy['operator'] : 'IN',
								);
							}
						}
					}


					$meta_queries = (array) get_post_meta($answerId, $this->prefix . 'meta_queries', true);	
					if(!empty( $this->array_filter_recursive($meta_queries) ) ) {

						foreach($meta_queries as $meta_query_data) {

							if(!isset($meta_query_data['meta_key']) || empty($meta_query_data['meta_key']) || !isset($meta_query_data['meta_value']) || empty($meta_query_data['meta_value'])) {
								continue;
							}

							if($filtering == "ranking") {

								if(!isset($meta_query[$questionId])) {
									$meta_query[$questionId] = array();
								}

							} else {

								if(!isset($meta_query[$questionId])) {

									$meta_query[$questionId] = array(
										'relation' => $relation,
									);
								}
							}


							if($filtering == "ranking") {

								$niceToHave[$questionId][$answerId] = array(
					                'key' => $meta_query_data['meta_key'],
					                'compare' => isset($meta_query_data['meta_operator']) ? $meta_query_data['meta_operator'] : '=',
					                'value' => $meta_query_data['meta_value'],
					                'type' => isset($meta_query_data['meta_type']) ? $meta_query_data['meta_type'] : 'CHAR',
								);

							} else {

								$meta_query[$questionId][] = array(
					                'key' => $meta_query_data['meta_key'],
					                'compare' => isset($meta_query_data['meta_operator']) ? $meta_query_data['meta_operator'] : '=',
					                'value' => $meta_query_data['meta_value'],
					                'type' => isset($meta_query_data['meta_type']) ? $meta_query_data['meta_type'] : 'CHAR',
								);

							}

						}
					}

					$matching_posts = get_post_meta($answerId, $this->prefix . 'posts', true);	
					if(!empty($matching_posts) && is_array($matching_posts)) {

						if(empty($post_in)) {
							$post_in = $matching_posts;
						} else {
							$post_in = array_intersect($matching_posts, $post_in);
							// $post_in = array_merge($matching_posts, $post_in);
						}
						
					}
				}
				
			}
		}

		$args['tax_query'] = $tax_query;
		$args['meta_query'] = $meta_query;
		$args['post__in'] = $post_in;
	
		$args = apply_filters( 'wordpress_form_wizard_posts_query_args', $args, $form_id );
       	$posts = get_posts( $args );
       	$posts = apply_filters( 'wordpress_form_wizard_posts', $posts, $form_id, $args, $choices_made );

		if($this->get_option('debugMode')) {
			var_dump($args);
			var_dump($posts);
		}

		if($this->get_option('wooCommerceVariationSupport') && !empty($this->get_option('wooCommerceVariationSupportAttributes')) && !empty($posts)) {

			$variationAttributes = explode(',', $this->get_option('wooCommerceVariationSupportAttributes') );
			foreach($args['tax_query'] as $a) {
				
				if(!is_array($a)) {
					continue;
				}

				foreach($a as $b) {

					if(!is_array($b)) {
						continue;
					}

					if(!isset($b['taxonomy'])) {
						continue;
					}

					$taxonomyExists = array_search($b['taxonomy'], $variationAttributes);
					if($taxonomyExists !== false) {

						$variationAttribute = $variationAttributes[$taxonomyExists];
						
						$term = get_term( reset($b['terms']), $variationAttribute);
						if(!$term) {
							continue;
						}

						foreach($posts as &$post) {

							$prod = wc_get_product($post->ID);
							if(!$prod) {
								continue;
							}

							if(!$prod->is_type('variable')) {
								continue;
							}

							$variationIds = $prod->get_children();
							foreach($variationIds as $variationId) {

								$variation = wc_get_product($variationId);
								if(!$variation) {
									continue;
								}

								$attributeValue = $variation->get_attribute($variationAttribute);
								if(empty($attributeValue)) {
									continue;
								}

								if($attributeValue != $term->name) {
									continue;
								}

								$post = get_post($variationId);
								break;
							}
						}
						break;
					}
				}
			}


		}

       	if(!empty($niceToHave) && !empty($posts)) {

       		$tmp = array();

       		foreach($posts as &$post) {

       			$labels = array();
       			$post->rankingFactor = 1;

	       		foreach($niceToHave as $niceToHaveKey => $niceToHaveValue) {

	       			foreach($niceToHaveValue as $niceToHaveValueKey => $niceToHaveValueSingle) {

	       				if($niceToHaveValueKey == 0) {

	       					if(isset($niceToHaveValueSingle['taxonomy'])) {
	       						$niceToHaveValueLabelText = strip_tags( get_the_term_list($post->ID, $niceToHaveValueSingle['taxonomy']) );
       						} else {
       							$niceToHaveValueLabelText = get_post_meta($post->ID, $niceToHaveValueSingle['key'], true);
       						}

	       				} else {
	       					$niceToHaveValuePost = get_post($niceToHaveValueKey);
	       					$niceToHaveValueLabelText = $niceToHaveValuePost->post_title;
	       				}

	       				if(isset($niceToHaveValueSingle['taxonomy'])) {
	       					$hasTerm = is_object_in_term( $post->ID, $niceToHaveValueSingle['taxonomy'], $niceToHaveValueSingle['terms'] );
	       					if($hasTerm) {

	       						$post->rankingFactor += 1;
	       						// foreach($niceToHaveValueSingle['terms'] as $niceToHaveValueSingleTerm) {

	       						// 	$term = get_term($niceToHaveValueSingleTerm, $niceToHaveValueSingle['taxonomy']);
	       						// 	if(!$term) {
	       						// 		continue;
	       						// 	}

	       						// 	$labels[] = '<span class="wordpress-form-wizard-posts-post-label">' . $term->name . '</span>';
	       						// }
	       						$labels[] = '<span class="wordpress-form-wizard-posts-post-label">' . $niceToHaveValueLabelText . '</span>'; //$niceToHaveValueSingle['value'];
	       						
	       					}

	       				} else {

	       					$meta = get_post_meta($post->ID, $niceToHaveValueSingle['key'], true);
	       					if(!empty($meta)) {	       						

	       						if(is_array($meta)) {

	       							foreach($meta as $singleMeta) {
	       								
	       								if (stripos( $singleMeta, $niceToHaveValueSingle['value']) !== false) {

	       									$post->rankingFactor += 1;
	       									$labels[] = '<span class="wordpress-form-wizard-posts-post-label">' . $niceToHaveValueLabelText . '</span>'; //$niceToHaveValueSingle['value'];
	       									break;
       									}
	       							}
	       						} else {
	       							// not working with slider <> custom value (do later)
		       						if (stripos($meta, $niceToHaveValueSingle['value']) !== false) {

		       							$post->rankingFactor += 1;
	       								$labels[] = '<span class="wordpress-form-wizard-posts-post-label">' . $niceToHaveValueLabelText . '</span>'; //$niceToHaveValueSingle['value'];
	       							}
       							}
	       					}

	       				}

	       			}
	       		}

	       		$post->labels = $labels;
       		}

       		// sort by array value rankingfactor
			usort($posts,function($first,$second){
			    return $first->rankingFactor < $second->rankingFactor;
			});
       	}
       	
       	if($this->get_option('orderby') == "lowestMidHighestPrice" && !empty($posts)) {
       		$tmp = array();

       		$maxPosts = count($posts) - 1;

       		$medianPost = round($maxPosts/2);
			
       		$tmp[] = $posts[0];
       		unset($posts[0]);

       		if($maxPosts > 1) {
	       		$tmp[] = $posts[$medianPost];
	       		unset($posts[$medianPost]);

	       		if($maxPosts > 2) {
		       		$tmp[] = $posts[$maxPosts];
		       		unset($posts[$maxPosts]);
		       	}						
			}

			$posts = array_merge($tmp, $posts);
       	}


		if($this->get_option('resultsBadges') && !empty($posts)) {

	       	$i = 1;
	       	foreach($posts as &$post) {

	       		if($i > 3) {
	       			break;
	       		}

	       		$post->badge = $this->get_option('resultsBadgesText' . $i);

	       		$i++;
       		}

		}

        $response['status'] = 1;
        $response['message'] = esc_html__('Posts found.', 'wordpress-form-wizard');
        $response['count'] = count($posts);
        $response['post_ids'] = wp_list_pluck($posts, 'ID');

        if(!empty($posts)) {
        	$response['html'] = $this->get_posts_template($posts);
        } else {
        	$response['html'] = '<div class="wordpress-form-wizard-no-results">' . $this->get_option('textsNoResults') . '</div>';
        }

        if($this->get_option('resultsLoadMore') && $response['count'] > $this->get_option('resultColumns')) {
			$response['html'] .= '<a href="#" class="btn button btn-default theme-button theme-btn wordpress-form-wizard-posts-load-more-button">' . $this->get_option('textsLoadMore') . '</a>';
        }

        echo json_encode($response);
        die();

    }

	public function array_filter_recursive($input) 
	{ 
		foreach ($input as &$value) 
		{ 
			if (is_array($value)) { 
				$value = $this->array_filter_recursive($value); 
			} 
		} 

		return array_filter($input); 
	} 

    public function get_posts_template($posts)
    {
		$this->resultData = $this->get_option('resultsData');
		$this->resultData = $this->resultData['enabled'];
		unset($this->resultData['placebo']);

		$this->resultDataIndex = 1;

    	$resultColumns = $this->get_option('resultColumns') ? $this->get_option('resultColumns') : 3;
    	$resultColumns = floor( 12 / $resultColumns ) ;

    	$html = '<div class="wordpress-form-wizard-row">';
    	foreach($posts as $post) {

			$style = "";
			if($this->get_option('resultsLoadMore') && $this->resultDataIndex > $this->get_option('resultColumns')) {
				$style = 'style="display: none;"';
			}

    		$html .= '<div class="wordpress-form-wizard-col-xs-6 wordpress-form-wizard-col-sm-' . $resultColumns . '" ' . $style . '>';
	    		
	    		$html .= $this->get_post_template($post);

			$html .= '</div>';

			$this->resultDataIndex += 1;
    	}
    	$html .= '</div>';

    	return $html;

    }

    public function get_post_template($post)
    {
    	$product = null;
		if($post->post_type == "product" || $post->post_type == "product_variation") {
			$product = wc_get_product($post->ID);
		}

    	$html = '<div class="wordpress-form-wizard-posts-post-container">';

    		if(array_key_first($this->resultData) == "im") {

				$data = $this->get_post_data($post, 'im', $product);

				$data = apply_filters( 'wordpress_form_wizard_post_data', $data, $post, 'im' );
				if(!empty($data)) {
					$html .= '<div class="wordpress-form-wizard-posts-post-image-container">';
						$html .= '<div class="wordpress-form-wizard-posts-post-im">' . $data . '</div>';
					$html .= '</div>';
				}				
    		}


    		$html .= '<div class="wordpress-form-wizard-posts-post-info-container">';

    			foreach ($this->resultData as $key => $value) {
					
					if($key == "im" && array_key_first($this->resultData) == "im") {
						continue;
					}

    				$data = $this->get_post_data($post, $key, $product);

					if(empty($data)) {
						continue;
					}

					if(strpos($key, 'attr') !== false) {
						$html .= '<div class="wordpress-form-wizard-posts-post-attr wordpress-form-wizard-posts-post-' . $key . '">' . $data . '</div>';
					} else {
						$html .= '<div class="wordpress-form-wizard-posts-post-' . $key . '">' . $data . '</div>';	
					}

					
				}

			$html .= '</div>';


    	$html .= '</div>';

		$html = apply_filters('wordpress_form_wizard_post_html', $html, $post, $post->post_type);

    	return $html;
    }

	protected function get_post_data($post, $key, $product = null)
	{
		$data = "";
		$post_id = $post->ID;

		$productBlacklistKeys = array(
			're',
			'va',
			'pr',
			'sk',
			'di',
			'we',
			'ca',
			'ta',
			'ct',
			'st',
		);

		if(!$product && in_array($key, $productBlacklistKeys)) {
			return $data;
		}

		$targetBlank = "";
		if($this->get_option('resultsOpenLinksInNewTab')) {
			$targetBlank = 'target="_blank"';
		}

		// Image
		if($key == "im") {
	        $link = get_permalink($post_id);
        	$img = wp_get_attachment_image_src( get_post_thumbnail_id( $post_id ), $this->get_option('imageSize') );
			$data =  (isset($img[0]) && !empty($img[0])) ?  $img[0] : $this->get_placeholder_image();
			$data = '<a ' . $targetBlank . ' href="' . $link . '"><img class="wordpress-form-wizard-responsive-image" src="' . $data . '" /></a>';

			if($this->get_option('resultsBadges') && isset($post->badge) && !empty($post->badge)) {
				$data = '<div class="wordpress-form-wizard-image-badge">' . $post->badge  . '</div>' . $data . '';
			}

		// Title
		} elseif($key == "ti") {

			if($product) {
				$data = $product->get_name();
			} else {
				$data = get_the_title($post_id);
			}

			if($this->get_option('titleLinkToProduct')) {
				$link = get_permalink($post_id);
				$data = '<a class="wordpress-form-wizard-title-link" ' . $targetBlank . ' href="' . $link . '">' . $data . '</a>';
			}


		// Ratings
		} elseif($key == "re") {
			$rating_count = $product->get_rating_count();
			$review_count = $product->get_review_count();
			$average      = $product->get_average_rating();
			$data = wc_get_rating_html($average, $rating_count);

		// Variations
		} elseif($key == "va") {
			if($product->is_type( 'variable' )) {
				$aval_variations = $product->get_available_variations();
				$variation_html = "";
				if(!empty($aval_variations)) {
					foreach ($aval_variations as $variation) {
						$variation_attributes = $variation['attributes'];
						foreach ($variation_attributes as $variation_attribute_key => $variation_attribute) {;
	                        if (strpos($variation_attribute_key, '_pa_')){ // variation is a pre-definted attribute
	                            $variation_attribute_key = substr($variation_attribute_key, 10);
	                            $attr = get_term_by('slug', $variation_attribute, $variation_attribute_key);
	                            $variation_attribute = $attr->name;
	                        } else { // variation is a custom attribute
	                            $attr = maybe_unserialize( get_post_meta( $post->ID, '_product_attributes' ) );
	                            
	                            $attr = get_term_by('slug', $variation_attribute, $variation_attribute_key);
	                            $variation_attribute = $attr->name;
	                        }
	                        $attr_label = wc_attribute_label($attr->taxonomy);
	                        $variation_html .= $attr_label . ': ' . $variation_attribute . '<br>';
                        }
					}
					$data = $variation_html;
				}
			}

		// Price
		} elseif($key == "pr") {
			$data = $product->get_price_html();

		// SKU
		} elseif($key == "sk") {
			$data = $product->get_sku();

			if($this->get_option('resultsShowAttributeName') && !empty($data)) {
				$data = sprintf( esc_html__('SKU: %s', 'wordpress-form-wizard'), $data);
			}

		// Excerpt (Short Description)
		} elseif($key == "ex") {

			if($product) {
				$data = $product->get_short_description();
			} else {
				$data = $post->excerpt;	
			}
			
			if($this->get_option('excerptStripShortcodes')) {
				$data = preg_replace("/\[[^\]]+\]/", '', $data);
			} else {
				$data = do_shortcode($data);
			}
		// Description
		} elseif($key == "de") {

			if($product) {
				$data = $product->get_description();
			} else {
				$data = $post->post_content;
			}

			if($this->get_option('excerptStripShortcodes')) {
				$data = preg_replace("/\[[^\]]+\]/", '', $data);
			} else {
				$data = do_shortcode($data);
			}
		// Dimensions
		} elseif($key == "di") {
			$data = wc_format_dimensions($product->get_dimensions(false));

			if($this->get_option('resultsShowAttributeName') && !empty($data)) {
				$data = sprintf( esc_html__('Dimensions: %s', 'wordpress-form-wizard'), $data);
			}

		// Weight
		} elseif($key == "we") {
			if(!empty($product->get_weight())) {
				$data = $product->get_weight() . get_option('woocommerce_weight_unit');
			}

			if($this->get_option('resultsShowAttributeName') && !empty($data)) {
				$data = sprintf( esc_html__('Weight: %s', 'wordpress-form-wizard'), $data);
			}

		// Add to Cart
		} elseif($key == "ca") {
 			ob_start();
			do_action( 'woocommerce_' . $product->get_type() . '_add_to_cart' );
	        $output_string = ob_get_contents();
	        ob_end_clean();
			$data = $output_string;

		// Tags
		} elseif($key == "ta") {
			$data = wc_get_product_tag_list($product->get_id());

		// Labels
		} elseif($key == "lb") {

			if(isset($post->labels) && is_array($post->labels)) {
				$data = implode(' ', $post->labels);
			}

		// Categories
		} elseif($key == "ct") {
			$data = $product->get_categories();


		// Read More
		} elseif($key == "lm") {
			$url = get_permalink($post_id);
			$data = '<a ' . $targetBlank . ' href="' . $url . '" class="btn button btn-default theme-button theme-btn wordpress-form-wizard-read-more">' . $this->get_option('textsLearnMore') . '</a>';
		
		// Get Stock Status
		} elseif($key == "st") {
			$data = $product->get_stock_status();

		// Get Attributes
		} elseif(strpos($key, 'attr') !== false) {

			if(!$product) {
				return $data;
			}

			$attribute_slug = substr($key, 5);
			$attribute_value = $product->get_attribute($attribute_slug);
			if(!empty($attribute_value)) {

				if($this->get_option('resultsShowAttributeName')) {
					$data = wc_attribute_label('pa_' . $attribute_slug) . ': ';
				}

				$data .= $attribute_value;
			}
		// Taxonomies
		} elseif(strpos($key, 'tx') !== false) {
			$tax_id = substr($key, -1);
			$taxonomy_name = $this->get_option('resultDataTaxonomy' . $tax_id);
			if($taxonomy_name) {
				$terms = get_the_term_list($post_id, $taxonomy_name, '', ', ');

				if(is_string($terms)) {

					$data = $terms;
					if($this->get_option('resultDataTaxonomyNoLinks'. $tax_id)) {
						$data = strip_tags($data);
					}

					$data = $this->get_option('resultDataTaxonomyPrefix' . $tax_id) . $data;
				}
			}
		// Custom Meta fields
		} elseif(strpos($key, 'mt') !== false) {
			$tax_id = substr($key, 2);
			$meta_key = $this->get_option('resultDataMeta' . $tax_id);

			if($meta_key) {
				
				$meta_data = get_post_meta($post_id, $meta_key, true);
				if(!empty($meta_data)) {
					$data = $this->get_option('resultDataMetaPrefix' . $tax_id) . $meta_data . $this->get_option('resultDataMetaSuffix' . $tax_id);
				}
			}
		}
		
		$data = apply_filters( 'wordpress_form_wizard_post_data', $data, $post, $key );

		return $data;
	}

	public function get_placeholder_image()
	{
		return plugin_dir_url( dirname( __FILE__ ) ) . 'assets/img/placeholder.png';
	}

    /**
     * Adds deposit value to price displays
     * e.g. archive, content, single
     *
     * @since   1.0.0
     */
    public function shortcode( $atts ) 
    {

        $args = shortcode_atts(array(
            'id' => '',
        ), $atts);

        $id = (int) $args['id'];
        if (empty($id)) {
            return esc_html__('No ID set.', 'wordpress-form-wizard');
        }

        $form = get_post($id);
        if(!$form) {
            return esc_html__('Form not found.', 'wordpress-form-wizard');
        }

        if($form->post_type != "wizard_form") {
            return esc_html__('Not a wizard form.', 'wordpress-form-wizard');
        }

        $formMeta = get_post_meta($id);

        $formType = $formMeta[$this->prefix . 'type'][0];
        $showEmail = isset($formMeta[$this->prefix . 'show_email'][0]) ? $formMeta[$this->prefix . 'show_email'][0] : 'yes';
        $showResultsAt = isset($formMeta[$this->prefix . 'show_results_at'][0]) ? $formMeta[$this->prefix . 'show_results_at'][0] : '';

        $questions = array();
        if(isset($formMeta[$this->prefix . 'questions']) && !empty($formMeta[$this->prefix . 'questions'][0])) {
        	$questionIds = unserialize($formMeta[$this->prefix . 'questions'][0]);
        	if(!empty($questionIds)) {

        		$tmp = array();
        		foreach($questionIds as $questionId) {

			        $question = get_post($questionId);
			        if(!$question) {
			            continue;
			        }

			        if($question->post_type != "wizard_question") {
			            continue;
			        }

			        if($question->post_status != "publish") {
			        	continue;
			        }

			        $questionMeta = get_post_meta($questionId);

			        $tmp[$questionId] = array_merge(
			        	$questionMeta,
			        	(array) $question,
			        );
        		}

        		$questions = $tmp;
        	}

    	}

    	$this->hasIntro = false;
		if( (isset($formMeta[$this->prefix . 'title']) && !empty($formMeta[$this->prefix . 'title'][0]) ) || (isset($formMeta[$this->prefix . 'intro']) && !empty($formMeta[$this->prefix . 'intro'][0])) ) { 
			$this->hasIntro = true;
		}

    	ob_start();

        ?>
    	<style>
			.rangeslider__handle, .rangeslider__fill {
				background-color: <?php echo $this->get_option('accentBackgroundColor') ?>;
			}

			.wordpress-form-wizard-posts-post-label,
			.wordpress-form-wizard-image-badge {
				background-color: <?php echo $this->get_option('accentBackgroundColor') ?>;
				color: <?php echo $this->get_option('accentTextColor') ?>;	
			}

			.wordpress-form-wizard-step-choices-container {
			    justify-content: <?php echo $this->get_option('textAlignment') ?>;
			}

			.wordpress-form-wizard-steps-container {
				text-align: <?php echo $this->get_option('textAlignment') ?>;
			}

			.wordpress-form-wizard-step-choice-container label,
			.wordpress-form-wizard-navigation,
			.wordpress-form-wizard-steps-container,
			.wordpress-form-wizard-top-bar {
			    border-color: <?php echo $this->get_option('borderColor') ?>;
			}

			.wordpress-form-wizard-step-choice-container :checked + label {
			    border-color: <?php echo $this->get_option('borderActiveColor') ?>;
			}

			<?php if($this->get_option('textAlignment') == "center") { ?>
				.wordpress-form-wizard-choice-no-image {
					text-align: left;
				}
			<?php }	?>

			<?php if($this->get_option('fixedHeight')) { ?>
				.wordpress-form-wizard-top-bar {
					height: <?php echo $this->get_option('fixedHeightTopBar'); ?>vh;
				}

				.wordpress-form-wizard-steps-container {
					height: <?php echo $this->get_option('fixedHeightStepsContainer'); ?>vh;
				}

				.wordpress-form-wizard-navigation {
					height: <?php echo $this->get_option('fixedHeightNavigation'); ?>vh;
				}

			<?php }	?>
			






    	</style>
		<form novalidate class="wordpress-form-wizard" 
			data-id="<?php echo $id ?>" 
			data-type="<?php echo $formType ?>" 
			data-show-results-at="<?php echo $showResultsAt ?>" 
			
			data-price-calculation="<?php echo htmlspecialchars( $formMeta[$this->prefix . 'price_calculation'][0]) ?>
		">


			<div class="wordpress-form-wizard-top-bar">

				<div class="wordpress-form-wizard-row">

					<?php if(isset($formMeta[$this->prefix . 'show_progress_bar']) && $formMeta[$this->prefix . 'show_progress_bar'][0] == "yes") { ?>

					<?php if($formType == "price_calculation" && isset($formMeta[$this->prefix . 'show_live_price']) && $formMeta[$this->prefix . 'show_live_price'][0] == "yes") { ?>
					<div class="wordpress-form-wizard-col-xs-6 wordpress-form-wizard-col-sm-10">
					<?php } else { ?>
					<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-12">
					<?php } ?>


						<div class="wordpress-form-wizard-progress-bar-container">
							<div class="wordpress-form-wizard-progress-bar" style="width: 0%; background-color: <?php echo $this->get_option('accentBackgroundColor') ?>;"></div>
						</div>
					</div>
					<?php } ?>

					<?php if($formType == "price_calculation" && isset($formMeta[$this->prefix . 'show_live_price']) && $formMeta[$this->prefix . 'show_live_price'][0] == "yes") { ?>
					<div class="wordpress-form-wizard-col-xs-6 wordpress-form-wizard-col-sm-2 wordpress-form-wizard-align-right">
						<div class="wordpress-form-wizard-price-container">
							<span class="wordpress-form-wizard-price-prefix"><?php echo esc_html( $this->get_option('textsPricePrefix') ) ?></span>
							<span class="wordpress-form-wizard-price" data-price="0">0</span>
							<span class="wordpress-form-wizard-price-suffix"><?php echo esc_html( $this->get_option('textsPriceSuffix') ) ?></span>
						</div>
					</div>
					<?php } ?>

				</div>
				
			</div>

			<div class="wordpress-form-wizard-steps-container">

				<?php if( $this->hasIntro ) { ?>
				<div class="wordpress-form-wizard-step wordpress-form-wizard-step-first wordpress-form-wizard-step-current" data-id="0">

					<div class="wordpress-form-wizard-row">
						<div class="wordpress-form-wizard-col-xs-12">

							<?php if(isset($formMeta[$this->prefix . 'title']) && !empty($formMeta[$this->prefix . 'title'][0])) { ?>
							<div class="wordpress-form-wizard-step-title"><?php echo esc_html($formMeta[$this->prefix . 'title'][0]) ?></div>
							<?php } ?>

							<?php if(isset($formMeta[$this->prefix . 'intro']) && !empty($formMeta[$this->prefix . 'intro'][0])) { ?>
							<div class="wordpress-form-wizard-step-description">
								<?php echo wpautop( do_shortcode( wp_kses( $formMeta[$this->prefix . 'intro'][0], $this->allowed_tags ) ) ) ?>
							</div>
							<?php } ?>

						</div>
					</div>
				</div>
				<?php } ?>

				<?php 
				if(!empty($questions)) { 
					foreach($questions as $question) {
						echo $this->get_question($question, $formMeta);
					}
				}
				?>

				<div class="wordpress-form-wizard-step wordpress-form-wizard-step-final" style="display: none;" data-id="0">

					<div class="wordpress-form-wizard-row">
						<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-center-xs">

							<?php if($formType == "price_calculation" && isset($formMeta[$this->prefix . 'show_final_price']) && $formMeta[$this->prefix . 'show_final_price'][0] == "yes") { ?>
							<div class="wordpress-form-wizard-step-final-price" 
							style="background-color: <?php echo $this->get_option('accentBackgroundColor') ?>; color: <?php echo $this->get_option('accentTextColor') ?>;"></div>
							<?php } ?>

							<?php if(isset($formMeta[$this->prefix . 'final_text']) && !empty($formMeta[$this->prefix . 'final_text'][0])) { ?>
							<div class="wordpress-form-wizard-step-description wordpress-form-wizard-step-description-final">
								<?php echo wpautop( do_shortcode( wp_kses( $formMeta[$this->prefix . 'final_text'][0], $this->allowed_tags ) ) ) ?>
							</div>
							<?php } ?>

						</div>
					</div>

					<?php if($this->get_option('email') && $showEmail == "yes") { ?>

					<div class="wordpress-form-wizard-submit-container">

						<div class="wordpress-form-wizard-row">

							<input class="wordpress-form-wizard-step-final-price-hidden" type="hidden" name="<?php echo $this->prefix ?>price" value="0">
							<input class="wordpress-form-wizard-step-final-post-ids-hidden" type="hidden" name="<?php echo $this->prefix ?>post_ids" value="0">
							
							<?php if($this->get_option('emailShowName')) { ?>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-6">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>firstname"><?php echo esc_html( $this->get_option('textsFirstname') ) ?></label>
									<input 
									id="<?php echo $this->prefix ?>firstname" 
									type="text" 
									<?php echo $this->get_option('emailShowNameRequired') ? 'required' : '' ?> 
									name="<?php echo $this->prefix ?>firstname" 
									placeholder="<?php echo esc_html( $this->get_option('textsFirstname') ) ?>">
								</div>
							</div>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-6">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>lastname"><?php echo esc_html( $this->get_option('textsLastname') ) ?></label>
									<input 
									id="<?php echo $this->prefix ?>lastname" 
									type="text" 
									<?php echo $this->get_option('emailShowNameRequired') ? 'required' : '' ?> 
									name="<?php echo $this->prefix ?>lastname" 
									placeholder="<?php echo esc_html( $this->get_option('textsLastname') ) ?>">
								</div>
							</div>
							<?php } ?>

							<?php if($this->get_option('emailShowCompany')) { ?>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-12">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>company"><?php echo esc_html( $this->get_option('textsCompany') ) ?></label>
									<input 
									id="<?php echo $this->prefix ?>company" 
									type="text" 
									<?php echo $this->get_option('companyShowCompanyRequired') ? 'required' : '' ?> 
									name="<?php echo $this->prefix ?>company" 
									placeholder="<?php echo esc_html( $this->get_option('textsCompany') ) ?>">
								</div>
							</div>
							<?php } ?>

							<?php if($this->get_option('emailShowEmail')) { ?>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-6">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>email"><?php echo esc_html( $this->get_option('textsEmail') ) ?></label>
									<input 
									id="<?php echo $this->prefix ?>email" 
									type="email" 
									<?php echo $this->get_option('emailShowEmailRequired') ? 'required' : '' ?> 
									name="<?php echo $this->prefix ?>email" 
									placeholder="<?php echo esc_html( $this->get_option('textsEmail') ) ?>">
								</div>
							</div>
							<?php } ?>

							<?php if($this->get_option('emailShowPhone')) { ?>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-6">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>phone"><?php echo esc_html( $this->get_option('textsPhone') ) ?></label>
									<input 
									id="<?php echo $this->prefix ?>phone" 
									type="tel" 
									<?php echo $this->get_option('emailShowPhoneRequired') ? 'required' : '' ?> 
									name="<?php echo $this->prefix ?>phone" 
									placeholder="<?php echo esc_html( $this->get_option('textsPhone') ) ?>">
								</div>
							</div>
							<?php } ?>	

							<?php if($this->get_option('emailShowAddress')) { ?>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-12">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>address_street"><?php echo esc_html( $this->get_option('textsAddressStreet') ) ?></label>
									<input 
									id="<?php echo $this->prefix ?>address_street" 
									type="text" 
									<?php echo $this->get_option('emailShowAddressRequired') ? 'required' : '' ?> 
									name="<?php echo $this->prefix ?>address_street" 
									placeholder="<?php echo esc_html( $this->get_option('textsAddressStreet') ) ?>">
								</div>
							</div>

							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-6">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>address_zip"><?php echo esc_html( $this->get_option('textsAddressZIP') ) ?></label>
									<input 
									id="<?php echo $this->prefix ?>address_zip" 
									type="text" 
									<?php echo $this->get_option('emailShowAddressRequired') ? 'required' : '' ?> 
									name="<?php echo $this->prefix ?>address_zip" 
									placeholder="<?php echo esc_html( $this->get_option('textsAddressZIP') ) ?>">
								</div>
							</div>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-6">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>address_city"><?php echo esc_html( $this->get_option('textsAddressCity') ) ?></label>
									<input 
									id="<?php echo $this->prefix ?>address_city" 
									type="text" 
									<?php echo $this->get_option('emailShowAddressRequired') ? 'required' : '' ?> 
									name="<?php echo $this->prefix ?>address_city" 
									placeholder="<?php echo esc_html( $this->get_option('textsAddressCity') ) ?>">
								</div>
							</div>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-12">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>address_country"><?php echo esc_html( $this->get_option('textsAddressCountry') ) ?></label>
									<input 
									id="<?php echo $this->prefix ?>address_country" 
									type="text" 
									<?php echo $this->get_option('emailShowAddressRequired') ? 'required' : '' ?> 
									name="<?php echo $this->prefix ?>address_country" 
									placeholder="<?php echo esc_html( $this->get_option('textsAddressCountry') ) ?>">
								</div>
							</div>
							<?php } ?>	

							<?php if($this->get_option('emailShowMessage')) { ?>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-12">
								<div class="wordpress-form-wizard-input-container">
									<label for="<?php echo $this->prefix ?>message"><?php echo esc_html( $this->get_option('textsMessage') ) ?></label>
									<textarea
										id="<?php echo $this->prefix ?>message" 
										<?php echo $this->get_option('emailShowMessageRequired') ? 'required' : '' ?> 
										name="<?php echo $this->prefix ?>message" 
										placeholder="<?php echo esc_html( $this->get_option('textsMessage') ) ?>"></textarea> 
								</div>
							</div>
							<?php } ?>	

							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-12">
								<div class="wordpress-form-wizard-input-container wordpress-form-wizard-input-container-privacy">
									<label for="<?php echo $this->prefix ?>privacy">
										<input name="<?php echo $this->prefix ?>privacy" id="<?php echo $this->prefix ?>privacy" type="checkbox" value="1">
										<?php echo $this->get_option('textsPrivacy'); ?>
									</label>
									
								</div>
							</div>

							<?php if(!empty($this->get_option('textsOptionalCheckbox'))) { ?>
							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-12">
								<div class="wordpress-form-wizard-input-container wordpress-form-wizard-input-container-optional-checkbox">
									<label for="<?php echo $this->prefix ?>optional_checkbox">
										<input name="<?php echo $this->prefix ?>optional_checkbox" id="<?php echo $this->prefix ?>optional_checkbox" type="checkbox" value="<?php echo esc_html( $this->get_option('textsOptionalCheckbox') ) ?>">
										<?php echo esc_html( $this->get_option('textsOptionalCheckbox') ); ?>
									</label>
								</div>
							</div>
							<?php } ?>

							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-12">
								<div class="wordpress-form-wizard-input-container wordpress-form-wizard-input-container-button">
									<input name="<?php echo $this->prefix ?>submit" class="wordpress-form-wizard-submit-button" type="submit" value="<?php echo esc_html( $this->get_option('textsSend') ) ?>" 
									style="background-color: <?php echo $this->get_option('accentBackgroundColor') ?>;">
								</div>
							</div>

						</div>
					</div>

					<?php } ?>

					<?php if($formType != "price_calculation") { ?>
					<div class="wordpress-form-wizard-posts-container">

					</div>
					<?php } ?>

					<div class="wordpress-form-wizard-loading-container" style="display: none;">
						<div class="wordpress-form-wizard-loading-icon">
							<i class="bx bx-radio-circle bx-burst" style="color: <?php echo $this->get_option('accentBackgroundColor') ?>;"></i>
						</div>
						<div class="wordpress-form-wizard-loading-text">
							<?php echo esc_html($this->get_option('textsLoadingText')) ?>
						</div>
					</div>

				</div>

			</div>

			<div class="wordpress-form-wizard-navigation">

				<button type="button" class="wordpress-form-wizard-btn wordpress-form-wizard-previous-step" disabled="disabled" style="display: none;"><?php echo esc_html( $this->get_option('textsBack') ) ?></button>

				
				<button type="button" class="wordpress-form-wizard-btn wordpress-form-wizard-next-step" 
					style="background-color: <?php echo $this->get_option('accentBackgroundColor') ?>; color: <?php echo $this->get_option('accentTextColor') ?>;"><?php echo esc_html( $this->get_option('textsNext') ) ?>
				</button>

				<?php if($this->get_option('showRestartButton')) { ?>
				<button type="button" class="wordpress-form-wizard-btn wordpress-form-wizard-restart" 
					style="display: none; background-color: <?php echo $this->get_option('accentBackgroundColor') ?>; color: <?php echo $this->get_option('accentTextColor') ?>;"><?php echo esc_html( $this->get_option('textsRestart') ) ?>
				</button>
				<?php } ?>

				<?php if($this->get_option('showSkipButton')) { ?>
				<button type="button" class="wordpress-form-wizard-btn wordpress-form-wizard-skip-step"><?php echo esc_html( $this->get_option('textsSkip') ) ?></button>
				<?php } ?>
			</div>
		</form>

		<?php
		$formHTML = apply_filters('wordpress_form_wizard_form_html', ob_get_contents(), $form);
		ob_end_clean();

		return $formHTML;
    }

    public function get_question($question, $formMeta) 
    {

    	if(!$question) {
    		return '';
    	}

    	$questionMeta = get_post_meta($question['ID']);

 		$answers = array();
        if(isset($questionMeta[$this->prefix . 'answers']) && !empty($questionMeta[$this->prefix . 'answers'][0])) {
        	$answerIds = unserialize($questionMeta[$this->prefix . 'answers'][0]);
        	if(!empty($answerIds)) {

        		$tmp = array();
        		foreach($answerIds as $answerId) {

			        $answer = get_post($answerId);
			        if(!$answer) {
			            continue;
			        }

			        if($answer->post_type != "wizard_answer") {
			            continue;
			        }

			        if($answer->post_status != "publish") {
			        	continue;
			        }

			        $answerMeta = get_post_meta($answerId);

			        $tmp[$answerId] = array_merge(
			        	$answerMeta,
			        	(array) $answer,
			        );
        		}

        		$answers = $tmp;
        	}
    	}

    	$columns = '4';
    	if(isset($questionMeta[$this->prefix . 'columns']) && !empty($questionMeta[$this->prefix . 'columns'][0])) {
    		$columns = ceil( 12 / $questionMeta[$this->prefix . 'columns'][0]);
		}

		$dependencies = array();
		if(isset($questionMeta[$this->prefix . 'dependencies']) && !empty($questionMeta[$this->prefix . 'dependencies'][0])) {
			$dependencies = unserialize( $questionMeta[$this->prefix . 'dependencies'][0] );
		}
		
		$extraClass = '" style="display: none;"';
		if(!$this->hasIntro) {
			$extraClass = 'wordpress-form-wizard-step-current"';
			$this->hasIntro = true;
		}

    	?>

		<div class="wordpress-form-wizard-step wordpress-form-wizard-step-<?php echo $question['ID'] ?> <?php echo $extraClass ?>" 
			data-dependency="<?php echo htmlspecialchars( json_encode($dependencies) ) ?>" data-id="<?php echo $question['ID'] ?>">


			<?php if(isset($question['post_title']) && !empty($question['post_title'])) { ?>
			<div class="wordpress-form-wizard-step-title"><?php echo esc_html( $question['post_title'] ) ?></div>
			<?php } ?>

			<?php if(isset($question[$this->prefix . 'description']) && !empty($question[$this->prefix . 'description'][0])) { ?>
			<div class="wordpress-form-wizard-step-description">
				<?php echo wpautop( do_shortcode( wp_kses( $question[$this->prefix . 'description'][0], $this->allowed_tags ) ) ) ?>
			</div>
			<?php } ?>

			<div class="wordpress-form-wizard-row wordpress-form-wizard-step-choices-container">

				<?php if(isset($questionMeta[$this->prefix . 'type'][0]) && $questionMeta[$this->prefix . 'type'][0] == "slider")  { ?>

					<div class="wordpress-form-wizard-col-xs-12">
						<div class="wordpress-form-wizard-rangeslider-container">
							<input
								name="<?php echo $question['ID'] ?>"
								class="wordpress-form-wizard-rangeslider"
							    type="range"
							    min="<?php echo $questionMeta[$this->prefix . 'slider_min'][0] ?>"
							    max="<?php echo $questionMeta[$this->prefix . 'slider_max'][0] ?>"                 
							    step="<?php echo $questionMeta[$this->prefix . 'slider_step'][0] ?>"                  
							    value="<?php echo $questionMeta[$this->prefix . 'slider_value'][0] ?>"                
							    data-price="<?php echo $questionMeta[$this->prefix . 'slider_price'][0] ?>"                
							    data-orientation="horizontal"
							    <?php echo $questionMeta[$this->prefix . 'required'][0] == "yes" ? 'required' : '' ?>
							>
							<div class="wordpress-form-wizard-rangeslider-info-container">
								<span class="wordpress-form-wizard-rangeslider-info-number"><?php echo $questionMeta[$this->prefix . 'slider_value'][0] ?></span>
								<span class="wordpress-form-wizard-rangeslider-info-metric"><?php echo $questionMeta[$this->prefix . 'slider_suffix'][0] ?></span>
							</div>
						</div>
					</div>

				<?php } elseif(isset($questionMeta[$this->prefix . 'type'][0]) && $questionMeta[$this->prefix . 'type'][0] == "textarea")  { ?>

					<div class="wordpress-form-wizard-col-xs-12">
						<div class="wordpress-form-wizard-rangeslider-container">
							<textarea
								name="<?php echo $question['ID'] ?>"
								class="wordpress-form-wizard-textarea"
							    <?php echo $questionMeta[$this->prefix . 'required'][0] == "yes" ? 'required' : '' ?>
							></textarea>
						</div>
					</div>

				<?php } else { ?>

					<?php if(!empty($answers)) { ?>

						<?php foreach($answers as $answer) { ?>

							<div class="wordpress-form-wizard-col-xs-12 wordpress-form-wizard-col-sm-<?php echo $columns ?>">

								<?php echo $this->get_answer($question, $answer, $questionMeta[$this->prefix . 'type'][0], $formMeta); ?>

							</div>

						<?php } ?>

					<?php } ?>

				<?php } ?>

			</div>
		</div>

		<?php
    }

    public function get_answer($question, $answer, $type = "radio", $formMeta = array() )
    {

    	if(!$answer) {
    		return '';
    	}
    	
    	$columns = '4';
    	if(isset($question[$this->prefix . 'columns']) && !empty($question[$this->prefix . 'columns'][0])) {
    		$columns = ceil( 12 / $question[$this->prefix . 'columns'][0]);
		}

    	$required = '';
    	if(isset($question[$this->prefix . 'required']) && $question[$this->prefix . 'required'][0] == "yes") {
    		$required = 'required';
		}

    	$style = 'image_top';
    	if(isset($question[$this->prefix . 'style']) ) {
    		$style = $question[$this->prefix . 'style'][0];
		}

		$image_src = "";
        if(isset($answer[$this->prefix . 'image']) && !empty($answer[$this->prefix . 'image'][0])) {
            $image_src_data = wp_get_attachment_image_src($answer[$this->prefix . 'image'][0], $this->get_option('imageSize'));
            if(isset($image_src_data[0])) {
                $image_src = $image_src_data[0];
            }
        }

		ob_start();

    	switch ($type) {
    		case 'checkbox':
    			?>
				<div class="wordpress-form-wizard-step-choice-container wordpress-form-wizard-step-choice-container-checkbox-style">
					<div class="wordpress-form-wizard-image-checkbox 
						<?php echo $style == "image_left" ? 'wordpress-form-wizard-choice-style-image-left' : 'wordpress-form-wizard-choice-style-image-top' ?>
						<?php echo empty($image_src) ? ' wordpress-form-wizard-choice-no-image' : '' ?>

						
					">	
						<input type="checkbox" name="<?php echo $question['ID'] ?>[]" <?php echo $required ?> id="wordpress_form_wizard_<?php echo $question['ID'] . $answer['ID'] ?>" value="<?php echo $answer['ID'] ?>" 
						data-price="<?php echo !empty($answer[$this->prefix . 'price'][0]) ? $answer[$this->prefix . 'price'][0] : 0 ?>" 
						/>
						<label for="wordpress_form_wizard_<?php echo $question['ID'] . $answer['ID'] ?>">
							<i class="bx bxs-check-square" style="color: <?php echo $this->get_option('accentBackgroundColor') ?>;"></i>
							<i class="bx bxs-square"></i>
							<div class="wordpress-form-wizard-row">

								<?php if(!empty($image_src)) { ?>

									<?php if($style == "image_left") { ?>
									<div class="wordpress-form-wizard-col-xs-5 wordpress-form-wizard-col-sm-3 wordpress-form-wizard-no-padding wordpress-form-wizard-step-choice-image-container">
									<?php } else { ?>
									<div class="wordpress-form-wizard-col-xs-5 wordpress-form-wizard-col-sm-12 wordpress-form-wizard-no-padding wordpress-form-wizard-step-choice-image-container">
									<?php } ?>
										<img class="wordpress-form-wizard-step-choice-img" src="<?php echo $image_src ?>" />
									</div>

								<?php } ?>

								<?php if($style == "image_left") { ?>
								<div class="wordpress-form-wizard-col-xs-7 wordpress-form-wizard-col-sm-9 wordpress-form-wizard-step-choice-title-container">
								<?php } else { ?>
								<div class="wordpress-form-wizard-col-xs-7 wordpress-form-wizard-col-sm-12 wordpress-form-wizard-step-choice-title-container">
								<?php } ?>

									<?php if(isset($answer['post_title']) && !empty($answer['post_title'])) { ?>
									<div class="wordpress-form-wizard-step-choice-title">
										
										<?php echo esc_html($answer['post_title']); ?>
										
										<?php if(isset($answer[$this->prefix . 'tooltip']) && !empty($answer[$this->prefix . 'tooltip'][0])) { ?>
										<span class="wordpress-form-wizard-tooltip" wordpress-form-wizard-tooltip="<?php echo esc_html($answer[$this->prefix . 'tooltip'][0]) ?>">
											<i class="bx bxs-info-circle"></i>
										</span>
										<?php } ?>

									</div>
									<?php } ?>

									<?php 
									if(

									 (isset($formMeta[$this->prefix . 'show_answer_prices']) && $formMeta[$this->prefix . 'show_answer_prices'][0] == "yes" ) && (isset($answer[$this->prefix . 'price']) && !empty($answer[$this->prefix . 'price'][0]) ) ) { ?>
									<div class="wordpress-form-wizard-step-choice-price-container">

										<span class="wordpress-form-wizard-price-prefix"><?php echo esc_html( $this->get_option('textsPricePrefix') ) ?></span>
										<span class="wordpress-form-wizard-choice-price"><?php echo esc_html($answer[$this->prefix . 'price'][0]) ?></span>
										<span class="wordpress-form-wizard-price-suffix"><?php echo esc_html( $this->get_option('textsPriceSuffix') ) ?></span>

									</div>
									<?php } ?>

									

									<?php if(isset($answer[$this->prefix . 'description']) && !empty($answer[$this->prefix . 'description'][0])) { ?>
									<div class="wordpress-form-wizard-step-choice-description">

										<div class="wordpress-form-wizard-step-description">
											<?php echo wpautop( do_shortcode( wp_kses( $answer[$this->prefix . 'description'][0], $this->allowed_tags ) ) ) ?>
										</div>
									</div>
									<?php } ?>

								</div>
							</div>
						</label>
					</div>
				</div>
				<?php 

    			break;
    		
    		default:
				?>

				<div class="wordpress-form-wizard-step-choice-container  wordpress-form-wizard-step-choice-container-radio-style">
					<div class="wordpress-form-wizard-image-checkbox 
						<?php echo $style == "image_left" ? 'wordpress-form-wizard-choice-style-image-left' : 'wordpress-form-wizard-choice-style-image-top' ?>
						<?php echo empty($image_src) ? ' wordpress-form-wizard-choice-no-image' : '' ?>
					">	
						<input type="radio" name="<?php echo $question['ID'] ?>" <?php echo $required ?> id="wordpress_form_wizard_<?php echo $question['ID'] . $answer['ID'] ?>" value="<?php echo $answer['ID'] ?>" data-price="<?php echo !empty($answer[$this->prefix . 'price'][0]) ? $answer[$this->prefix . 'price'][0] : 0 ?>"
						/>
						<label for="wordpress_form_wizard_<?php echo $question['ID'] . $answer['ID'] ?>">
							<i class="bx bxs-check-circle" style="color: <?php echo $this->get_option('accentBackgroundColor') ?>;"></i>
							<i class="bx bxs-circle"></i>
							<div class="wordpress-form-wizard-row">

								<?php if(!empty($image_src)) { ?>

									<?php if($style == "image_left") { ?>
									<div class="wordpress-form-wizard-col-xs-4 wordpress-form-wizard-col-sm-3 wordpress-form-wizard-no-padding wordpress-form-wizard-step-choice-image-container">
									<?php } else { ?>
									<div class="wordpress-form-wizard-col-xs-4 wordpress-form-wizard-col-sm-12 wordpress-form-wizard-no-padding wordpress-form-wizard-step-choice-image-container">
									<?php } ?>
									
										<img class="wordpress-form-wizard-step-choice-img" src="<?php echo $image_src ?>" />
									</div>
								<?php } ?>

								<?php if($style == "image_left") { ?>
								<div class="wordpress-form-wizard-col-xs-8 wordpress-form-wizard-col-sm-9 wordpress-form-wizard-step-choice-title-container">
								<?php } else { ?>
								<div class="wordpress-form-wizard-col-xs-8 wordpress-form-wizard-col-sm-12 wordpress-form-wizard-step-choice-title-container">
								<?php } ?>

									<?php if(isset($answer['post_title']) && !empty($answer['post_title'])) { ?>
									<div class="wordpress-form-wizard-step-choice-title">
										
										<?php echo esc_html($answer['post_title']); ?>
										
										<?php if(isset($answer[$this->prefix . 'tooltip']) && !empty($answer[$this->prefix . 'tooltip'][0])) { ?>
										<span class="wordpress-form-wizard-tooltip" wordpress-form-wizard-tooltip="<?php echo esc_html($answer[$this->prefix . 'tooltip'][0]) ?>">
											<i class="bx bxs-info-circle"></i>
										</span>
										<?php } ?>

									</div>
									<?php } ?>

									<?php 
									if(

									 (isset($formMeta[$this->prefix . 'show_answer_prices']) && $formMeta[$this->prefix . 'show_answer_prices'][0] == "yes" ) && (isset($answer[$this->prefix . 'price']) && !empty($answer[$this->prefix . 'price'][0]) ) ) { ?>
									<div class="wordpress-form-wizard-step-choice-price-container">

										<span class="wordpress-form-wizard-price-prefix"><?php echo esc_html( $this->get_option('textsPricePrefix') ) ?></span>
										<span class="wordpress-form-wizard-choice-price"><?php echo esc_html($answer[$this->prefix . 'price'][0]) ?></span>
										<span class="wordpress-form-wizard-price-suffix"><?php echo esc_html( $this->get_option('textsPriceSuffix') ) ?></span>

									</div>
									<?php } ?>

									<?php if(isset($answer[$this->prefix . 'description']) && !empty($answer[$this->prefix . 'description'][0])) { ?>
									<div class="wordpress-form-wizard-step-choice-description">

										<div class="wordpress-form-wizard-step-description">
											<?php echo wpautop( do_shortcode( wp_kses( $answer[$this->prefix . 'description'][0], $this->allowed_tags ) ) ) ?>
										</div>
									</div>
									<?php } ?>

								</div>
							</div>
						</label>
					</div>
				</div>

				<?php
    			break;
    	}

		$answerHTML = apply_filters('wordpress_form_wizard_answer_html', ob_get_contents(), $question, $answer, $type);
		ob_end_clean();

		return $answerHTML;
    }
}